
$fn = 60;

include <Estes.scad>
include <rocketry.scad>

//------------------------------------------------------------------

module jog (h = -0.001) {
	translate ([0, 0, h])
		children ();
};

//------------------------------------------------------------------
// Functions to extract segment height and total height from h vectors
// If zero element is 0 then the vector is assumed to be offsets from zero
// otherwise they are segment heights

module stack (a, s)
{
	// if the segment is zero then the stack offset is always zero
	if (s == 0) {
		children ();
	}
	else {
		// If the first element is nonzero then the elements are segment
		// heights so add them up inductively
		if (a[0] > 0) {
			stack (a, s - 1)
				translate ([0, 0, a[s - 1]])
					children ();
		}
		else {
			translate ([0, 0, a[s]])
				children ();
		}
	}
}

function height (a, s) =
	a[0] > 0 ? a[s] : a[s + 1] - a[s]; 

//------------------------------------------------------------------

module pnc_60mx ()
{
	h = [
		0.500,
		0.050,
		1.332,
		0.792,
		0.665,
	];

	d = [
		[id,			id],
		[BT60[OD],	BT60[OD]],
		[BT60[OD],	1.227],
		[1.227,		0.775],
	];	
	
	for (i = [0 : len (d) - 1]) {
		stack (h, i)
			cylinder (h=height (h, i) + 0.001, d1=d[i][0], d2=d[i][1]);
	}

	tip_r = 0.300/2;

	stack (h, 4)
	sb_cone (h[4] - tip_r, d[3][1]/2, tip_r);
}

//-------------------------------------------------------------------------------------

module booster_nose ()
{
	h = [
		0.500,
		0.050,
		1.368
	];

	d = [
		[BT50[ID],	BT50[ID]],
		[BT50[OD],	BT50[OD]]
	];	
	
	for (i = [0 : len (d) - 1]) {
		stack (h, i)
			cylinder (h=height (h, i) + 0.001, d1=d[i][0], d2=d[i][1]);
	}

	tip_r = 0.175/2;

	stack (h, 2)
	sb_cone (h[2] - tip_r, d[1][1]/2, tip_r);
}

//-------------------------------------------------------------------------------------
// Broken up for printing

module booster_tail1 ()
{
	difference () {
		rotate ([0, 180, 0]) jog (-0.355 - 0.585) // Flip
		intersection () {
			booster_tail ();
			jog ()
			cylinder (h = 0.355 + 0.585, d = 2);
		}

		// Alignment pin
		jog ()
			cylinder (h = 0.25, d = 0.500);
	}			
}

module booster_tail2 ()
{
	difference () {
		intersection () {
			jog (-0.355 - 0.585 - 0.001)
				booster_tail ();
			jog ()
				cylinder (h = 0.355 + 0.585, d = 2);
		}

		// Alignment pin
		jog ()
			cylinder (h = 0.25, d = 0.500);
	}			
}
	
//-------------------------------------------------------------------------------------

module booster_tail ()
{
	h = [
		0.355,
		0.000,
		0.585,
		0.000,
		0.075,
		0.030,
		0.500
	];

	d = [
		[0.921,		0.721],
		[0.721,		1.253],
		[1.253,		1.000],
		[1.000,		1.250],
		[1.250,		1.250],
		[1.252,		0.950],
		[0.950,		0.950]		
	];	

	tab_w = 0.088;
	v_thick = 0.060;

	union () {
		difference () {
			for (i = [0 : len (d) - 1]) {
				stack (h, i)
					cylinder (h=height (h, i) + 0.000, d1=d[i][0], d2=d[i][1]);
			}

			// Tail cone void
			jog ()
			cylinder (h = h[0], d1 = d[0][0] - 2*v_thick, d2 = d[0][1] - 2*v_thick);
		}

		// Tab
		stack (h, 3)
		translate ([0.475, 0, -0.280])
	
		rotate ([0, 6, 0])
		union () {
			translate ([0, 0, 0.150])
				cube ([tab_w, tab_w, 0.300], center = true);
			translate ([tab_w/2, 0, 0])
				cylinder (h = 0.300, d = tab_w);
		}
	}
}

//------------------------------------------------------------------

module trapezoid (h, b, t, l)
{
	points = [
		[-b/2, 0],
		[-t/2, h],
		[t/2, h],
		[b/2, 0]
	];

	linear_extrude (height = l)
		polygon (points = points);
}

//------------------------------------------------------------------

module rounded_cylinder (h, r, cap1 = true, cap2 = true)
{
	assign ($fn = 16) {
		union () {
			if (cap1) jog (r) sphere (r = r);
			jog (h/2) cylinder (h = h - 2  * r, r = r, center = true);
			if (cap2) jog (h - r) sphere (r = r);
		}
	}
}

//------------------------------------------------------------------

module hemisphere (r)
{
	difference () {
		sphere (r, center = true);
		translate ([0, 0, -r/2])
			cube ([2 * r, 2 * 4, r], center = true);
	}
}

//------------------------------------------------------------------
// Transition section of satellite

module satellite1 ()
{
	h = [
		0.975,
		0.000,
		1.265,
		0.060,
		0.000,
		0.375,
		0.250,

		0.250,
	];

	d = [
		[BT55[ID],	BT55[ID]],
		[BT55[ID],	BT55[OD]],
		[BT55[OD],	BT60[OD]],
		[BT60[OD],	BT60[OD]],
		[BT60[OD],	BT60[ID]],
		[BT60[ID],	BT60[ID]],
		[BT60[ID],	0.700],

		// alignment pin
		[0.500,		0.500]	
	];	

	difference () {
		union () {
			// Circular stack	
			for (i = [0 : len (d) - 1]) {
				stack (h, i)
					cylinder (h=height (h, i) + 0.001, d1=d[i][0], d2=d[i][1]);
			}
			
			// friction ribs to allow for adjusting the slip fit
			stack (h, 0)
			for (a = [0 : 60 : 300]) {
				rotate (a)
				translate ([ BT55[ID]/2 - 0.010, 0, h[0] * 1/8])
					rounded_cylinder (h = h[0] * 6/8, r = 0.020);
			}
		}

		// attachement point
		jog ()
		attachment_point (BT55[ID]/2);
	}
}

//------------------------------------------------------------------

module satellite2 ()
{
	h = [
		0.050,
		0.375,
		0.125,
		0.160,
		1.075
	];

	d = [
		[0.700,		0.785],
		[0.785,		0.600],
		[0.600,		1.050],
		[1.050,		1.125],
		[1.125,		1.125]
	];	

	// Circular stack
	difference () {
		for (i = [0 : len (d) - 1]) {
			assign (slop = i < len (d) - 1 ? 0.001 : 0.0) 
				stack (h, i)
					cylinder (h=height (h, i) + slop, d1=d[i][0], d2=d[i][1]);
		}

		jog ()
		cylinder (h = 0.250, d= 0.500 + slip);

		stack (h, 5) jog (-0.250 + 0.002)
		cylinder (h = 0.350, d= 0.500 + slip);
	}

	// Compute values for trapezoid section
	r = 1.125/2;
	thick = 0.375;
	top = 0.750;
	line_r = 0.020;

	// Compute the legth of the base
	base = r * sqrt (3);

	// compute the triangle angle
	angle = atan (thick / (base/2 - top/2));

	// Trapezoid offset from centerline
	y = base/2 * tan (30) - 0.001;

	// spherical tanks.  radius computed from exposed measurements
	tank_h = 0.100;
	tank_w = 0.445;
	tank_r = (tank_h * tank_h + tank_w * tank_w / 4)/ (2 * tank_h);
	box_h = height (h, 4) * 1/5;
	box_w = height (h, 4) * 1/6;

	echo (box_h, box_w);

	stack (h, 4) union () {
		// Spheres
		rotate (0) translate ([0, y, 0])
			union () {
				trapezoid (thick, base, top, height (h, 4));

				union () {
					translate ([-0.085, thick - tank_r + tank_h, 0.30])
					rotate ([-90, 0, 0]) hemisphere (r = tank_r);
		
					translate ([0.085, thick - tank_r + tank_h, height (h, 4) - 0.30])
					rotate ([-90, 0, 0]) hemisphere (r = tank_r);
				}

				// Spheres right side
				translate ([-top/2, thick, 0.00])
					rotate (angle)
						translate ([-0.380 - line_r, 0, 0.140]) rotate ([0, 90, 0])
							rounded_cylinder (h = 0.270, r = line_r);
					
			}

		// Dual boxes
		rotate (120) translate ([0, y, 0])
			union () {
				trapezoid (thick, base, top, height (h, 4));

				translate ([0, thick, 0.07]) {
					translate ([0.2, 0, 0.8])
					cube ([0.250, 0.050, 0.275], center = true);
		
					translate ([-0.2, 0, 0.2])
					cube ([0.150, 0.050, 0.333], center = true);
				}				

				// Dual boxes left side
				translate ([top/2, thick, 0.00])
					rotate (180-angle) {
						translate ([-0.260, 0, 0.255 - line_r])
							cylinder (h = 0.840, r = line_r);
						translate ([-0.370 - line_r, 0, 0.240]) rotate ([0, 90, 0])
							rounded_cylinder (h = 0.150, r = line_r);
		
						translate ([-0.150, 0, 0.155 - line_r])
							cylinder (h = 0.940, r = line_r);
						translate ([-0.400 - line_r, 0, 0.140]) rotate ([0, 90, 0])
							rounded_cylinder (h = 0.290, r = line_r);
					}

				// Right side
				translate ([-top/2, thick, 0.00])
					rotate (angle) {
						translate ([-0.240, 0, 0.240 - line_r])
							rounded_cylinder (h = 0.80, r = line_r);
						translate ([-0.320 - line_r, 0, 0.240]) rotate ([0, 90, 0])
							cylinder (h = 0.100, r = line_r);
						translate ([-0.340, 0, 0.200 - line_r])
							rounded_cylinder (h = 0.080, r = line_r);
		
						translate ([-0.150, 0, 0.190 - line_r])
							rounded_cylinder (h = 0.85, r = line_r, cap1=false);
						translate ([-0.250 - line_r, 0, 0.190]) rotate ([0, 90, 0])
							rounded_cylinder (h = 0.140, r = line_r);
		
					}

			}


		// Box
		rotate (180) translate ([0, y + 0.24, box_h/2])
		cube ([box_w, box_w, box_h], center = true);

		// Cross
		rotate (240) translate ([0, y, 0]) {
			// Cross
			union () {
				trapezoid (thick, base, top, height (h, 4));

				translate ([0, thick, 0.07]) {
					translate ([-.3, 0, 0])
					rotate ([0, 33, 0])
					rounded_cylinder (h = 1.1, r = 0.035);
			
					translate ([.3, 0, 0])
					rotate ([0, -33, 0])
					rounded_cylinder (h = 1.1, r = 0.035);
				}		

				// Left side
				translate ([top/2, thick, 0.0])
					rotate (180-angle) {
						translate ([-0.15, 0, 0.92])
							cube ([0.210, 0.05, 0.210], center=true);
						translate ([-0.115, 0, 0.67])
							cube ([0.135, 0.05, 0.135], center=true);
					}
	
	
				// Cross right side
				translate ([-top/2, thick, 0.00])
					rotate (angle) {
						translate ([-0.280, 0, 0.120 - line_r])
							rounded_cylinder (h = 0.97, r = line_r);
						translate ([-0.280 - line_r, 0, 0.120]) rotate ([0, 90, 0])
							rounded_cylinder (h = 0.190, r = line_r, cap1=false);
		
						translate ([-0.200, 0, 0.550 - line_r])
							rounded_cylinder (h = 0.54, r = line_r);
						translate ([-0.180 - line_r, 0, 0.550]) rotate ([0, 90, 0])
							cylinder (h = 0.100, r = line_r);
						translate ([-0.095, 0, 0.240 - line_r])
							rounded_cylinder (h = 0.350, r = line_r);
						translate ([-0.205 - line_r, 0, 0.240]) rotate ([0, 90, 0])
							rounded_cylinder (h = 0.150, r = line_r, cap2=false);
		
						translate ([-0.180, 0, 0.225/2 + 0.280])
							cube ([0.085, 0.05, 0.225], center=true);
					}
			}
		}
	}	
}

//------------------------------------------------------------------

module satellite3 ()
{	
	height = 1.660;
	
	// Square section
	difference () {
		union () {
			difference ()
			{
				jog (height/2)
				cube ([1.360, 0.870, height], center = true);
		
				// Chamfers
				rotate (45) translate ([0, 1.20, 0])
					cube ([1, 1, 5], center = true);
		
				rotate (135) translate ([0, 1.20, 0])
					cube ([1, 1, 5], center = true);
		
				rotate (45) translate ([0, -1.20, 0])
					cube ([1, 1, 5], center = true);
		
				rotate (135) translate ([0, -1.20, 0])
					cube ([1, 1, 5], center = true);
			}
	
			translate ([-0.975/2, 0, 0])
			cube ([0.975, 0.525, height]);
	
			translate ([-0.975/2, 0, 0.090])
			cube ([0.975, 0.565, height - 0.090]);

			// Small transition to ease printing.  Not canonical
			translate ([-0.975/2, 0.425, 0.090])
			rotate ([-45, 0, 0])
			cube ([0.975, 0.090, 0.090]);
	
			translate ([1.000/2, 0, 0])	rotate (180)
			union () {
				cube ([1.000, 0.525, height]);

				translate ([0, 0, 0.090])
				cube ([1.000, 0.565, height - 0.090]);

				// Small transition to ease printing.  Not canonical
				translate ([0, 0.425, 0.090])
				rotate ([-45, 0, 0])
				cube ([1.000, 0.090, 0.090]);
			}
		}
	
		//alignment hole
		jog ()
		cylinder (h = 0.250, d = 0.500);	
	}	

	// Top post
	h = [
		0.220,
		0.275
	];

	d = [
		[0.550,		0.320],
		[0.320,		0.320]

	];	

	jog (height - 0.001) {
		translate ([-0.530-0.200/2, -0.200, 0])
		cube ([0.200, 0.200, 0.085]);

		translate ([0.530-0.250/2, -0.350/2, 0])
		cube ([0.250, 0.350, 0.085]);

		// Circular stack	
		for (i = [0 : len (d) - 1]) {
			stack (h, i)
				cylinder (h=height (h, i) + 0.001, d1=d[i][0], d2=d[i][1]);
		}
	}
}

//-------------------------------------------------------------------------------------

slip = 0.005;
detail = 0;

id = BT60[ID] - slip;
od = BT60[OD];

coll		= 0.250;
angle	= 45;

// Jog heights for assembly
h1 = [
	18 - 0.975,
	2.9,
	1.8,
	3,
	13.3
];

h2 = [
	0,
	19,
	22.5,
	25.,
	28.0
];

h = h1;


stack (h, 0) color ("tan")
tube (h = 18, d = BT55[OD], t = (BT55[OD] - BT55[ID])/2);

color ("white") {
	stack (h, 1)
	satellite1 ();

	stack (h, 2)
//	rotate ([180, 0, 0]) // This is just for rendering for printing
	satellite2 ();

	stack (h, 3)
	satellite3 ();

	stack (h, 4)
	pnc_60mx ();
}

for (a = [0:1]) {
	rotate (a * 180) translate ([BT55[OD]/2 + BT50[OD]/2 + 0.080, 0, 0]) {
		color ("tan")
		tube (h = 9.50, d = BT50[OD], t = (BT55[OD] - BT55[ID])/2);

		color ("white") {
			jog (9.000)
!				booster_nose ();
			jog (-1.03) {
				booster_tail ();
			}
		}
	}
}
